@@ -4,6 +4,7 @@
 #' @param names  character vector of region ids to be added to the neighbours list
 #' 
 #' @return Spatial djacency matrix.
+#' @importFrom spdep poly2nb nb2mat
 #' @author Zehang Richard Li 
 #' @examples
 #' \dontrun{
@@ -16,6 +17,7 @@
 
 
 getAmat <- function(geo, names){
+	if("sf" %in% class(geo)) rownames(geo) <- names
 	nb.r <- spdep::poly2nb(geo, queen=F, row.names = names)
 	mat <- spdep::nb2mat(nb.r, style="B",zero.policy=TRUE)
 	regions <- colnames(mat) <- rownames(mat) 
